@@ -1,29 +1,38 @@
 # Arithmetic tests for DataFrame/Series/Index/Array classes that should
 # behave identically.
 # Specifically for numeric dtypes
+from __future__ import annotations
+
 from collections import abc
 from decimal import Decimal
 from itertools import combinations
 import operator
-from typing import Any, List
+from typing import Any
 
 import numpy as np
 import pytest
 
 import pandas as pd
 from pandas import (
-    Float64Index,
     Index,
-    Int64Index,
     RangeIndex,
     Series,
     Timedelta,
     TimedeltaIndex,
-    UInt64Index,
     array,
 )
 import pandas._testing as tm
 from pandas.core import ops
+from pandas.core.api import (
+    Float64Index,
+    Int64Index,
+    UInt64Index,
+)
+from pandas.core.computation import expressions as expr
+from pandas.tests.arithmetic.common import (
+    assert_invalid_addsub_type,
+    assert_invalid_comparison,
+)
 
 
 @pytest.fixture(params=[Index, Series, tm.to_array])
@@ -49,11 +58,22 @@ def adjust_negative_zero(zero, expected):
     return expected
 
 
+def compare_op(series, other, op):
+    left = np.abs(series) if op in (ops.rpow, operator.pow) else series
+    right = np.abs(other) if op in (ops.rpow, operator.pow) else other
+
+    cython_or_numpy = op(left, right)
+    python = left.combine(right, op)
+    if isinstance(other, Series) and not other.index.equals(series.index):
+        python.index = python.index._with_freq(None)
+    tm.assert_series_equal(cython_or_numpy, python)
+
+
 # TODO: remove this kludge once mypy stops giving false positives here
 # List comprehension has incompatible type List[PandasObject]; expected List[RangeIndex]
 #  See GH#29725
-ser_or_index: List[Any] = [Series, Index]
-lefts: List[Any] = [RangeIndex(10, 40, 10)]
+ser_or_index: list[Any] = [Series, Index]
+lefts: list[Any] = [RangeIndex(10, 40, 10)]
 lefts.extend(
     [
         cls([10, 20, 30], dtype=dtype)
@@ -79,25 +99,13 @@ class TestNumericComparisons:
         expected = 0.0 > Series([1, 2, 3])
         tm.assert_series_equal(result, expected)
 
-    def test_df_numeric_cmp_dt64_raises(self):
+    def test_df_numeric_cmp_dt64_raises(self, box_with_array, fixed_now_ts):
         # GH#8932, GH#22163
-        ts = pd.Timestamp.now()
-        df = pd.DataFrame({"x": range(5)})
+        ts = fixed_now_ts
+        obj = np.array(range(5))
+        obj = tm.box_expected(obj, box_with_array)
 
-        msg = (
-            "'[<>]' not supported between instances of 'numpy.ndarray' and 'Timestamp'"
-        )
-        with pytest.raises(TypeError, match=msg):
-            df > ts
-        with pytest.raises(TypeError, match=msg):
-            df < ts
-        with pytest.raises(TypeError, match=msg):
-            ts < df
-        with pytest.raises(TypeError, match=msg):
-            ts > df
-
-        assert not (df == ts).any().any()
-        assert (df != ts).all().all()
+        assert_invalid_comparison(obj, ts, box_with_array)
 
     def test_compare_invalid(self):
         # GH#8058
@@ -112,12 +120,12 @@ class TestNumericComparisons:
         box = box_with_array
         xbox = box if box is not Index else np.ndarray
 
-        obj = Series(np.random.randn(10 ** 5))
+        obj = Series(np.random.randn(10**5))
         obj = tm.box_expected(obj, box, transpose=False)
 
         result = obj == "a"
 
-        expected = Series(np.zeros(10 ** 5, dtype=bool))
+        expected = Series(np.zeros(10**5, dtype=bool))
         expected = tm.box_expected(expected, xbox, transpose=False)
         tm.assert_equal(result, expected)
 
@@ -134,8 +142,6 @@ class TestNumericComparisons:
 
 
 class TestNumericArraylikeArithmeticWithDatetimeLike:
-
-    # TODO: also check name retentention
     @pytest.mark.parametrize("box_cls", [np.array, Index, Series])
     @pytest.mark.parametrize(
         "left", lefts, ids=lambda x: type(x).__name__ + str(x.dtype)
@@ -155,7 +161,6 @@ class TestNumericArraylikeArithmeticWithDatetimeLike:
         result = right * left
         tm.assert_equal(result, expected)
 
-    # TODO: also check name retentention
     @pytest.mark.parametrize("box_cls", [np.array, Index, Series])
     @pytest.mark.parametrize(
         "left", lefts, ids=lambda x: type(x).__name__ + str(x.dtype)
@@ -219,13 +224,18 @@ class TestNumericArraylikeArithmeticWithDatetimeLike:
         ],
         ids=lambda x: type(x).__name__,
     )
-    def test_numeric_arr_mul_tdscalar_numexpr_path(self, scalar_td, box_with_array):
+    @pytest.mark.parametrize("dtype", [np.int64, np.float64])
+    def test_numeric_arr_mul_tdscalar_numexpr_path(
+        self, dtype, scalar_td, box_with_array
+    ):
+        # GH#44772 for the float64 case
         box = box_with_array
 
-        arr = np.arange(2 * 10 ** 4).astype(np.int64)
+        arr_i8 = np.arange(2 * 10**4).astype(np.int64, copy=False)
+        arr = arr_i8.astype(dtype, copy=False)
         obj = tm.box_expected(arr, box, transpose=False)
 
-        expected = arr.view("timedelta64[D]").astype("timedelta64[ns]")
+        expected = arr_i8.view("timedelta64[D]").astype("timedelta64[ns]")
         expected = tm.box_expected(expected, box, transpose=False)
 
         result = obj * scalar_td
@@ -262,62 +272,38 @@ class TestNumericArraylikeArithmeticWithDatetimeLike:
             np.timedelta64("NaT", "D"),
             pd.offsets.Minute(3),
             pd.offsets.Second(0),
-        ],
-    )
-    def test_add_sub_timedeltalike_invalid(self, numeric_idx, other, box_with_array):
-        box = box_with_array
-
-        left = tm.box_expected(numeric_idx, box)
-        msg = (
-            "unsupported operand type|"
-            "Addition/subtraction of integers and integer-arrays|"
-            "Instead of adding/subtracting|"
-            "cannot use operands with types dtype|"
-            "Concatenation operation is not implemented for NumPy arrays"
-        )
-        with pytest.raises(TypeError, match=msg):
-            left + other
-        with pytest.raises(TypeError, match=msg):
-            other + left
-        with pytest.raises(TypeError, match=msg):
-            left - other
-        with pytest.raises(TypeError, match=msg):
-            other - left
-
-    @pytest.mark.parametrize(
-        "other",
-        [
-            pd.Timestamp.now().to_pydatetime(),
-            pd.Timestamp.now(tz="UTC").to_pydatetime(),
-            pd.Timestamp.now().to_datetime64(),
+            # GH#28080 numeric+datetimelike should raise; Timestamp used
+            #  to raise NullFrequencyError but that behavior was removed in 1.0
+            pd.Timestamp("2021-01-01", tz="Asia/Tokyo"),
+            pd.Timestamp("2021-01-01"),
+            pd.Timestamp("2021-01-01").to_pydatetime(),
+            pd.Timestamp("2021-01-01", tz="UTC").to_pydatetime(),
+            pd.Timestamp("2021-01-01").to_datetime64(),
+            np.datetime64("NaT", "ns"),
             pd.NaT,
         ],
     )
-    @pytest.mark.filterwarnings("ignore:elementwise comp:DeprecationWarning")
-    def test_add_sub_datetimelike_invalid(self, numeric_idx, other, box_with_array):
-        # GH#28080 numeric+datetime64 should raise; Timestamp raises
-        #  NullFrequencyError instead of TypeError so is excluded.
+    def test_add_sub_datetimedeltalike_invalid(
+        self, numeric_idx, other, box_with_array
+    ):
         box = box_with_array
-        left = tm.box_expected(numeric_idx, box)
 
+        left = tm.box_expected(numeric_idx, box)
         msg = "|".join(
             [
                 "unsupported operand type",
-                "Cannot (add|subtract) NaT (to|from) ndarray",
                 "Addition/subtraction of integers and integer-arrays",
+                "Instead of adding/subtracting",
+                "cannot use operands with types dtype",
                 "Concatenation operation is not implemented for NumPy arrays",
+                "Cannot (add|subtract) NaT (to|from) ndarray",
                 # pd.array vs np.datetime64 case
                 r"operand type\(s\) all returned NotImplemented from __array_ufunc__",
+                "can only perform ops with numeric values",
+                "cannot subtract DatetimeArray from ndarray",
             ]
         )
-        with pytest.raises(TypeError, match=msg):
-            left + other
-        with pytest.raises(TypeError, match=msg):
-            other + left
-        with pytest.raises(TypeError, match=msg):
-            left - other
-        with pytest.raises(TypeError, match=msg):
-            other - left
+        assert_invalid_addsub_type(left, other, msg)
 
 
 # ------------------------------------------------------------------
@@ -387,9 +373,14 @@ class TestDivisionByZero:
     # ------------------------------------------------------------------
 
     @pytest.mark.parametrize("dtype1", [np.int64, np.float64, np.uint64])
-    def test_ser_div_ser(self, dtype1, any_real_dtype):
+    def test_ser_div_ser(
+        self,
+        switch_numexpr_min_elements,
+        dtype1,
+        any_real_numpy_dtype,
+    ):
         # no longer do integer div for any ops, but deal with the 0's
-        dtype2 = any_real_dtype
+        dtype2 = any_real_numpy_dtype
 
         first = Series([3, 4, 5, 8], name="first").astype(dtype1)
         second = Series([0, 0, 0, 3], name="second").astype(dtype2)
@@ -401,15 +392,20 @@ class TestDivisionByZero:
                 name=None,
             )
         expected.iloc[0:3] = np.inf
+        if first.dtype == "int64" and second.dtype == "float32":
+            # when using numexpr, the casting rules are slightly different
+            # and int64/float32 combo results in float32 instead of float64
+            if expr.USE_NUMEXPR and switch_numexpr_min_elements == 0:
+                expected = expected.astype("float32")
 
         result = first / second
         tm.assert_series_equal(result, expected)
         assert not result.equals(second / first)
 
     @pytest.mark.parametrize("dtype1", [np.int64, np.float64, np.uint64])
-    def test_ser_divmod_zero(self, dtype1, any_real_dtype):
+    def test_ser_divmod_zero(self, dtype1, any_real_numpy_dtype):
         # GH#26987
-        dtype2 = any_real_dtype
+        dtype2 = any_real_numpy_dtype
         left = Series([1, 1]).astype(dtype1)
         right = Series([0, 2]).astype(dtype2)
 
@@ -532,13 +528,25 @@ class TestDivisionByZero:
     # ------------------------------------------------------------------
     # Mod By Zero
 
-    def test_df_mod_zero_df(self):
+    def test_df_mod_zero_df(self, using_array_manager):
         # GH#3590, modulo as ints
         df = pd.DataFrame({"first": [3, 4, 5, 8], "second": [0, 0, 0, 3]})
-
         # this is technically wrong, as the integer portion is coerced to float
-        # ###
-        first = Series([0, 0, 0, 0], dtype="float64")
+        first = Series([0, 0, 0, 0])
+        if not using_array_manager:
+            # INFO(ArrayManager) BlockManager doesn't preserve dtype per column
+            # while ArrayManager performs op column-wisedoes and thus preserves
+            # dtype if possible
+            first = first.astype("float64")
+        second = Series([np.nan, np.nan, np.nan, 0])
+        expected = pd.DataFrame({"first": first, "second": second})
+        result = df % df
+        tm.assert_frame_equal(result, expected)
+
+        # GH#38939 If we dont pass copy=False, df is consolidated and
+        #  result["first"] is float64 instead of int64
+        df = pd.DataFrame({"first": [3, 4, 5, 8], "second": [0, 0, 0, 3]}, copy=False)
+        first = Series([0, 0, 0, 0], dtype="int64")
         second = Series([np.nan, np.nan, np.nan, 0])
         expected = pd.DataFrame({"first": first, "second": second})
         result = df % df
@@ -665,11 +673,10 @@ class TestMultiplicationDivision:
         tm.assert_series_equal(result, expected)
 
     def test_mul_index(self, numeric_idx):
-        # in general not true for RangeIndex
         idx = numeric_idx
-        if not isinstance(idx, RangeIndex):
-            result = idx * idx
-            tm.assert_index_equal(result, idx ** 2)
+
+        result = idx * idx
+        tm.assert_index_equal(result, idx**2)
 
     def test_mul_datelike_raises(self, numeric_idx):
         idx = numeric_idx
@@ -768,7 +775,7 @@ class TestMultiplicationDivision:
         df = pd.DataFrame({"A": ts})
 
         tm.assert_series_equal(ts + ts, ts + df["A"], check_names=False)
-        tm.assert_series_equal(ts ** ts, ts ** df["A"], check_names=False)
+        tm.assert_series_equal(ts**ts, ts ** df["A"], check_names=False)
         tm.assert_series_equal(ts < ts, ts < df["A"], check_names=False)
         tm.assert_series_equal(ts / ts, ts / df["A"], check_names=False)
 
@@ -857,7 +864,7 @@ class TestAdditionSubtraction:
         tm.assert_frame_equal(second + first, expected)
 
     # TODO: This came from series.test.test_operators, needs cleanup
-    def test_series_frame_radd_bug(self):
+    def test_series_frame_radd_bug(self, fixed_now_ts):
         # GH#353
         vals = Series(tm.rands_array(5, 10))
         result = "foo_" + vals
@@ -873,13 +880,19 @@ class TestAdditionSubtraction:
         ts.name = "ts"
 
         # really raise this time
-        now = pd.Timestamp.now().to_pydatetime()
-        msg = "unsupported operand type"
+        fix_now = fixed_now_ts.to_pydatetime()
+        msg = "|".join(
+            [
+                "unsupported operand type",
+                # wrong error message, see https://github.com/numpy/numpy/issues/18832
+                "Concatenation operation",
+            ]
+        )
         with pytest.raises(TypeError, match=msg):
-            now + ts
+            fix_now + ts
 
         with pytest.raises(TypeError, match=msg):
-            ts + now
+            ts + fix_now
 
     # TODO: This came from series.test.test_operators, needs cleanup
     def test_datetime64_with_index(self):
@@ -957,77 +970,54 @@ class TestAdditionSubtraction:
                 assert (df + df).equals(df)
                 tm.assert_frame_equal(df + df, df)
 
-    # TODO: taken from tests.series.test_operators; needs cleanup
-    def test_series_operators(self):
-        def _check_op(series, other, op, pos_only=False):
-            left = np.abs(series) if pos_only else series
-            right = np.abs(other) if pos_only else other
+    @pytest.mark.parametrize(
+        "func",
+        [lambda x: x * 2, lambda x: x[::2], lambda x: 5],
+        ids=["multiply", "slice", "constant"],
+    )
+    def test_series_operators_arithmetic(self, all_arithmetic_functions, func):
+        op = all_arithmetic_functions
+        series = tm.makeTimeSeries().rename("ts")
+        other = func(series)
+        compare_op(series, other, op)
 
-            cython_or_numpy = op(left, right)
-            python = left.combine(right, op)
-            if isinstance(other, Series) and not other.index.equals(series.index):
-                python.index = python.index._with_freq(None)
-            tm.assert_series_equal(cython_or_numpy, python)
+    @pytest.mark.parametrize(
+        "func", [lambda x: x + 1, lambda x: 5], ids=["add", "constant"]
+    )
+    def test_series_operators_compare(self, comparison_op, func):
+        op = comparison_op
+        series = tm.makeTimeSeries().rename("ts")
+        other = func(series)
+        compare_op(series, other, op)
 
-        def check(series, other):
-            simple_ops = ["add", "sub", "mul", "truediv", "floordiv", "mod"]
+    @pytest.mark.parametrize(
+        "func",
+        [lambda x: x * 2, lambda x: x[::2], lambda x: 5],
+        ids=["multiply", "slice", "constant"],
+    )
+    def test_divmod(self, func):
+        series = tm.makeTimeSeries().rename("ts")
+        other = func(series)
+        results = divmod(series, other)
+        if isinstance(other, abc.Iterable) and len(series) != len(other):
+            # if the lengths don't match, this is the test where we use
+            # `tser[::2]`. Pad every other value in `other_np` with nan.
+            other_np = []
+            for n in other:
+                other_np.append(n)
+                other_np.append(np.nan)
+        else:
+            other_np = other
+        other_np = np.asarray(other_np)
+        with np.errstate(all="ignore"):
+            expecteds = divmod(series.values, np.asarray(other_np))
 
-            for opname in simple_ops:
-                _check_op(series, other, getattr(operator, opname))
+        for result, expected in zip(results, expecteds):
+            # check the values, name, and index separately
+            tm.assert_almost_equal(np.asarray(result), expected)
 
-            _check_op(series, other, operator.pow, pos_only=True)
-
-            _check_op(series, other, ops.radd)
-            _check_op(series, other, ops.rsub)
-            _check_op(series, other, ops.rtruediv)
-            _check_op(series, other, ops.rfloordiv)
-            _check_op(series, other, ops.rmul)
-            _check_op(series, other, ops.rpow, pos_only=True)
-            _check_op(series, other, ops.rmod)
-
-        tser = tm.makeTimeSeries().rename("ts")
-        check(tser, tser * 2)
-        check(tser, tser[::2])
-        check(tser, 5)
-
-        def check_comparators(series, other):
-            _check_op(series, other, operator.gt)
-            _check_op(series, other, operator.ge)
-            _check_op(series, other, operator.eq)
-            _check_op(series, other, operator.lt)
-            _check_op(series, other, operator.le)
-
-        check_comparators(tser, 5)
-        check_comparators(tser, tser + 1)
-
-    # TODO: taken from tests.series.test_operators; needs cleanup
-    def test_divmod(self):
-        def check(series, other):
-            results = divmod(series, other)
-            if isinstance(other, abc.Iterable) and len(series) != len(other):
-                # if the lengths don't match, this is the test where we use
-                # `tser[::2]`. Pad every other value in `other_np` with nan.
-                other_np = []
-                for n in other:
-                    other_np.append(n)
-                    other_np.append(np.nan)
-            else:
-                other_np = other
-            other_np = np.asarray(other_np)
-            with np.errstate(all="ignore"):
-                expecteds = divmod(series.values, np.asarray(other_np))
-
-            for result, expected in zip(results, expecteds):
-                # check the values, name, and index separately
-                tm.assert_almost_equal(np.asarray(result), expected)
-
-                assert result.name == series.name
-                tm.assert_index_equal(result.index, series.index._with_freq(None))
-
-        tser = tm.makeTimeSeries().rename("ts")
-        check(tser, tser * 2)
-        check(tser, tser[::2])
-        check(tser, 5)
+            assert result.name == series.name
+            tm.assert_index_equal(result.index, series.index._with_freq(None))
 
     def test_series_divmod_zero(self):
         # Check that divmod uses pandas convention for division by zero,
@@ -1056,11 +1046,11 @@ class TestUFuncCompat:
         box = Series if holder is Series else Index
 
         if holder is RangeIndex:
-            idx = RangeIndex(0, 5)
+            idx = RangeIndex(0, 5, name="foo")
         else:
-            idx = holder(np.arange(5, dtype="int64"))
+            idx = holder(np.arange(5, dtype="int64"), name="foo")
         result = np.sin(idx)
-        expected = box(np.sin(np.arange(5, dtype="int64")))
+        expected = box(np.sin(np.arange(5, dtype="int64")), name="foo")
         tm.assert_equal(result, expected)
 
     @pytest.mark.parametrize("holder", [Int64Index, UInt64Index, Float64Index, Series])
@@ -1178,14 +1168,16 @@ class TestNumericArithmeticUnsorted:
     def check_binop(self, ops, scalars, idxs):
         for op in ops:
             for a, b in combinations(idxs, 2):
+                a = a._rename("foo")
+                b = b._rename("bar")
                 result = op(a, b)
                 expected = op(Int64Index(a), Int64Index(b))
-                tm.assert_index_equal(result, expected)
+                tm.assert_index_equal(result, expected, exact="equiv")
             for idx in idxs:
                 for scalar in scalars:
                     result = op(idx, scalar)
                     expected = op(Int64Index(idx), scalar)
-                    tm.assert_index_equal(result, expected)
+                    tm.assert_index_equal(result, expected, exact="equiv")
 
     def test_binops(self):
         ops = [
@@ -1212,7 +1204,7 @@ class TestNumericArithmeticUnsorted:
         idxs = [RangeIndex(0, 10, 1), RangeIndex(0, 20, 2)]
         self.check_binop(ops, scalars, idxs)
 
-    # TODO: mod, divmod?
+    # TODO: divmod?
     @pytest.mark.parametrize(
         "op",
         [
@@ -1222,6 +1214,7 @@ class TestNumericArithmeticUnsorted:
             operator.floordiv,
             operator.truediv,
             operator.pow,
+            operator.mod,
         ],
     )
     def test_arithmetic_with_frame_or_series(self, op):
@@ -1276,19 +1269,23 @@ class TestNumericArithmeticUnsorted:
 
         # __pow__
         idx = RangeIndex(0, 1000, 2)
-        result = idx ** 2
-        expected = idx._int64index ** 2
+        result = idx**2
+        expected = Int64Index(idx._values) ** 2
         tm.assert_index_equal(Index(result.values), expected, exact=True)
 
         # __floordiv__
         cases_exact = [
             (RangeIndex(0, 1000, 2), 2, RangeIndex(0, 500, 1)),
             (RangeIndex(-99, -201, -3), -3, RangeIndex(33, 67, 1)),
-            (RangeIndex(0, 1000, 1), 2, RangeIndex(0, 1000, 1)._int64index // 2),
+            (
+                RangeIndex(0, 1000, 1),
+                2,
+                Int64Index(RangeIndex(0, 1000, 1)._values) // 2,
+            ),
             (
                 RangeIndex(0, 100, 1),
                 2.0,
-                RangeIndex(0, 100, 1)._int64index // 2.0,
+                Int64Index(RangeIndex(0, 100, 1)._values) // 2.0,
             ),
             (RangeIndex(0), 50, RangeIndex(0)),
             (RangeIndex(2, 4, 2), 3, RangeIndex(0, 1, 1)),
@@ -1364,18 +1361,42 @@ def test_integer_array_add_list_like(
     left = container + box_1d_array(data)
     right = box_1d_array(data) + container
 
-    if Series == box_pandas_1d_array:
-        assert_function = tm.assert_series_equal
-        expected = Series(expected_data, dtype="Int64")
-    elif Series == box_1d_array:
-        assert_function = tm.assert_series_equal
-        expected = Series(expected_data, dtype="object")
-    elif Index in (box_pandas_1d_array, box_1d_array):
-        assert_function = tm.assert_index_equal
-        expected = Int64Index(expected_data)
+    if Series in [box_1d_array, box_pandas_1d_array]:
+        cls = Series
+    elif Index in [box_1d_array, box_pandas_1d_array]:
+        cls = Index
     else:
-        assert_function = tm.assert_numpy_array_equal
-        expected = np.array(expected_data, dtype="object")
+        cls = array
 
-    assert_function(left, expected)
-    assert_function(right, expected)
+    expected = cls(expected_data, dtype="Int64")
+
+    tm.assert_equal(left, expected)
+    tm.assert_equal(right, expected)
+
+
+def test_sub_multiindex_swapped_levels():
+    # GH 9952
+    df = pd.DataFrame(
+        {"a": np.random.randn(6)},
+        index=pd.MultiIndex.from_product(
+            [["a", "b"], [0, 1, 2]], names=["levA", "levB"]
+        ),
+    )
+    df2 = df.copy()
+    df2.index = df2.index.swaplevel(0, 1)
+    result = df - df2
+    expected = pd.DataFrame([0.0] * 6, columns=["a"], index=df.index)
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("power", [1, 2, 5])
+@pytest.mark.parametrize("string_size", [0, 1, 2, 5])
+def test_empty_str_comparison(power, string_size):
+    # GH 37348
+    a = np.array(range(10**power))
+    right = pd.DataFrame(a, dtype=np.int64)
+    left = " " * string_size
+
+    result = right == left
+    expected = pd.DataFrame(np.zeros(right.shape, dtype=bool))
+    tm.assert_frame_equal(result, expected)
