@@ -27,7 +27,13 @@ pytestmark = pytest.mark.usefixtures("pyarrow_skip")
 
 
 @pytest.mark.network
-@tm.network
+@tm.network(
+    url=(
+        "https://raw.github.com/pandas-dev/pandas/main/"
+        "pandas/tests/io/parser/data/salaries.csv"
+    ),
+    check_before_test=True,
+)
 def test_url(all_parsers, csv_dir_path):
     parser = all_parsers
     kwargs = {"sep": "\t"}
